#![allow(unreachable_code)]

use gif::{DecodeOptions, Frame};
use std::fs::OpenOptions;
use std::process::exit;
use x11rb::connection::Connection;
use x11rb::protocol::xproto::ConnectionExt;
use x11rb::protocol::xproto::CreateGCAux;

use std::collections::VecDeque;

struct Exit<'a>{
    conn: &'a x11rb::rust_connection::RustConnection,
    pixmap: u32,
    gc: u32,
}
impl Drop for Exit<'_>{
    fn drop(&mut self)
    {
        self.conn.free_gc(self.gc).unwrap();
        self.conn.free_pixmap(self.pixmap).unwrap();

    }
}

fn main() {
    let args = std::env::args();
    if args.len() < 2
    {
        print!("nice try dink!");
        exit(1);
    }

    let arg = args.skip(1).next().unwrap();
    let input = OpenOptions::new().read(true).open(arg).unwrap(); // Grabs image from arguement

    let mut de_options = DecodeOptions::new();
    de_options.set_color_output(gif::ColorOutput::RGBA);
    let mut decoder = de_options.read_info(input).unwrap();

    let height = decoder.height();
    let width = decoder.width();

    let (conn, screen_num) = x11rb::connect(None).unwrap(); //connects to xserver
    let screen = &conn.setup().roots[screen_num]; //gets the main screen
    let root = screen.root; //grabs the root ID of screen
    let depth = screen.root_depth; //grabs depth of root window

    let pixmap = conn.generate_id().unwrap(); //generates an ID to use for pixel data presumably
    conn.create_pixmap(depth, pixmap, root, width, height)
        .unwrap(); //makes a space for pixel data on
                   //root depth with pixmap ID at
                   //400x400 space, need to make
                   //larger

    let gc = conn.generate_id().unwrap(); //generates another ID for Graphics Context
    let gc_aux = CreateGCAux::new().foreground(screen.white_pixel); // create aux for connecting the
                                                                    // context to thingy
    conn.create_gc(gc, root, &gc_aux).unwrap();
    
    let _exit: Exit = Exit {
        conn: &conn,
        pixmap,
        gc
    };
    //Decoder Shenanigans

    let mut v_frames: VecDeque<Frame> = VecDeque::new();
    while let Some(frame) = decoder.read_next_frame().unwrap() {
        v_frames.push_back(frame.clone());
    } //Putting gif frames into a vector
    //println!("{:?}", v_frames.get(0));
    println!("Done!");
    loop {
        let popd = &v_frames.pop_front().unwrap();

        let put = conn
            .put_image(
                x11rb::protocol::xproto::ImageFormat::Z_PIXMAP,
                root,
                gc,
                width,
                height,
                0,
                0,
                0,
                depth,
                &popd.buffer,
            )
            .unwrap();
        let copy = conn
            .copy_area(root, pixmap, gc, 0, 0, 0, 0, width, height)
            .unwrap();
        std::thread::sleep(std::time::Duration::from_millis(50));
        v_frames.push_back(popd.clone());

        put.check().unwrap();
        copy.check().unwrap();
    }
    conn.flush().unwrap();

    // unreachable but whatever
    drop(_exit);
}
